/*
 * ar3cmonitor - meterwidget.cpp
 *
 * Copyright (C) 2004  Szymon Acedanski <accek@podlesie.net>
 * Copyright (C) 2006..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <qpainter.h>
#include <cmath>

#if QT_VERSION >= 0x40000
#include <QResizeEvent>
#endif

#include "fontsize_resolver.h"
#include "niceformat.h"
#include "meterwidget.h"
#include "control.h"
#include <cdata.h>
#include <fonts.h>
#include <ar3cm_proto.h>

MeterVWidget::MeterVWidget(int _analogIndex, QColor _color, float _maxValue,
		float _minValue, float _unitValue, unsigned int flags,
		prec_t _prec,
		QWidget *parent, const char *name, Qt::WFlags f)
	: QWidget(parent, name, f), prec(_prec)
{
	unsigned int size;

	setMinimumHeight(30);
	size = ar3cmon_font_size(AR3CMONITOR_FONT_VALUE);
	val_font.setPointSize(size);
	setFont(val_font);
	char buf[32];
	str_niceformat(buf, 32, _unitValue, _minValue, _maxValue, prec);
	val_format = QString(buf);

	setMinimumWidth(QFontMetrics(val_font).width(val_format) * 6 / 5);

	setFocusPolicy(QT_STRONG_FOCUS);
#if QT_VERSION >= 0x30000
	setBackgroundMode(Qt::NoBackground);
#else
	setBackgroundMode(NoBackground);
#endif
	maxValue = _maxValue;
	minValue = _minValue;
	unitValue = _unitValue;
	color = _color;
	analogIndex = _analogIndex;
	oldvalid = 0;
	meter_flags = flags;
}

QString MeterVWidget::niceFormat(float v)
{
	char buf[32];

	niceformat(buf, sizeof(buf), unitValue, minValue, maxValue, v, prec);
	return QString(buf);
}

void MeterVWidget::paintEvent(QPaintEvent *)
{
	QPainter p(this);
	int text_h, w_h;
	int ret;
	int iv;

	if (meter_flags & AR3CM_VMETER_ALONG)
		ret = cdata_get_along(analogIndex, &iv);
	else
		ret = cdata_get_analog(analogIndex, &iv);

	if (ret) {
		p.setPen(Qt::NoPen);
		p.setBrush(Qt::black);
		p.drawRect(0, 0, width(), height());
		p.setPen(Qt::red);
		p.drawText(0, 0, width(), height(), Qt::AlignCenter,
				QString("?"));
		oldvalid = 0;
		return;
	}
	float v = iv / unitValue;

	oldvalid = 1;
	oldValue = v;

	p.setFont(val_font);
	text_h = QFontMetrics(val_font).height();
	w_h = height() - (text_h + 5);

	int level = (int)((1.0 - (v - minValue) / (maxValue - minValue))
			* (w_h-2) + 1);
	int zero_level = (int)((1.0 + minValue / (maxValue - minValue))
			* (w_h-2) + 1);

	if (zero_level > w_h - 1)
		zero_level = w_h - 1;
	if (level > w_h - 1)
		level = w_h - 1;
	p.setPen(Qt::NoPen);
	p.setBrush(Qt::black);
	{
		int ml = (level > zero_level) ? level : zero_level;
		int mh = (level < zero_level) ? level : zero_level;

		p.drawRect(QRect(QPoint(1, 0), QPoint(width() - 2, mh)));
		p.drawRect(QRect(QPoint(1, ml), QPoint(width() - 2, height()
						- 1)));
	}

	p.setPen(color);
	p.setBrush(color);
	p.drawRect(QRect(QPoint(1, level), QPoint(width() - 2, zero_level)));

	p.setPen(Qt::white);
	p.setBrush(Qt::white);
	p.drawRect(QRect(QPoint(1, zero_level - 1), QPoint(width() - 2,
					zero_level + 1)));

	p.drawRect(QRect(QPoint(1, w_h - 1), QPoint(width() - 2, w_h + 1)));
	p.drawText(0, height() - (text_h + 3), width() - 1, text_h,
			Qt::AlignHCenter | Qt::AlignBottom | Qt::SingleLine,
			niceFormat(v));

	p.setPen(Qt::white);
	p.setBrush(Qt::NoBrush);

#if QT_VERSION >= 0x40000
	p.drawRect(0, 0, width() - 1, height() - 1);
#else
	p.drawRect(0, 0, width(), height());
#endif
}

void MeterVWidget::updateMeter()
{
	int need_update = 0;
	int iv;

	if (cdata_get_analog(analogIndex, &iv)) {
		if (oldvalid)
			update();
		return;
	}

	if (oldvalid) {
		float v = iv / unitValue;

		if (v != oldValue)
			need_update = 1;
	} else
		need_update = 1;

	if (need_update)
		update();
}


MeterLabel::MeterLabel(const QString &caption, QColor _color, QWidget *parent,
		const char *name, Qt::WFlags f)
	: QWidget(parent, name, f)
{
	setMinimumHeight(16);
	setMinimumWidth(20);
	setBackgroundColor(Qt::black);
	color = _color;
	text = caption;
}

void MeterLabel::paintEvent(QPaintEvent *)
{
	QPainter p(this);

	p.setPen(color);
	p.drawText(0, 2, width(), height() - 4, Qt::AlignHCenter
			| Qt::AlignVCenter, text);
}

void MeterLabel::setLabel(const QString &s)
{
	text = s;
	update();
}
