/*
 * AR-3c Monitor
 *
 * 	version 2.4
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * See COPYING for license information
 *
 */

#include <qlabel.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <cmath>
#include <unistd.h>

#include "object.h"
#include "commobj.h"
#include "control.h"
#include "ar3cmonitor.h"

#if UNIX && !defined(__SIMPAD__)
#define XP_UNIX

typedef uint16_t uint16;
typedef uint32_t uint32;

typedef struct _NPRect
{
    uint16	top;
    uint16	left;
    uint16	bottom;
    uint16	right;
} NPRect;

typedef struct _NPWindow 
{
    void*	window;		/* Platform specific window handle */
    uint32	x;			/* Position of top left corner relative */
    uint32	y; 			/*	to a netscape page.					*/
    uint32	width;		/* Maximum window size */
    uint32	height;
    NPRect	clipRect;	/* Clipping rectangle in port coordinates */
						/* Used by MAC only.                      */
#ifdef XP_UNIX
    void *	ws_info;	/* Platform-dependent additonal data */
#endif /* XP_UNIX */
} NPWindow;
#endif

ObjectWidget::ObjectWidget(const char *command, QWidget *parent,
		const char *name, Qt::WFlags f)
	: QWidget(parent, name, f)
{
	w = -1;
	h = -1;
#if UNIX && !defined(__SIMPAD__)
	char opts[256];
	int ret;
	int p[2];

	infofd = -1;
	ret = pipe(p);
	if (ret == -1)
		return;
	infofd = p[1];
	snprintf(opts, 256, "%d,%d,%d,%ld,%d,%d,%d,%d",
			2576, 1, p[0], winId(), 0, 0, 0, 0);
	ret = fork();
	if (!ret) {
		::close(p[1]);
		execlp("aramon-object", "aramon-object", opts, command, NULL);
		_exit(EXIT_FAILURE);
	}
	::close(p[0]);
#endif
}

void ObjectWidget::paintEvent(QPaintEvent *)
{
#if UNIX && !defined(__SIMPAD__)
	int nw = width();
	int nh = height();
	NPWindow wininfo;
	ssize_t ret;
	size_t written = 0;

	if (nw == w && nh == h)
		return;
	w = nw;
	h = nh;
	memset(&wininfo, 0, sizeof(wininfo));
	wininfo.window = (void *) (unsigned long) winId();
	wininfo.width = w;
	wininfo.height = h;

	/* FIXME: This can fail */
	while (written < sizeof(wininfo)) {
		ret = write(infofd, (char *) &wininfo + written, sizeof(wininfo) - written);
		if (ret <= 0)
			break;
		written += ret;
	}
#endif
}

ObjectWidget::~ObjectWidget()
{
	::close(infofd);
}
