/*
 * ar3cmonitor - test-tslist.cpp
 *
 * Copyright (C) 2006..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#define HAVE_INTTYPES_H
#define HAVE_SYS_TIME_H
#define HAVE_LOCALTIME_R
#define HAVE_STRNDUP

#include "timestampedlist.h"
#include "datastore.h"
#include <stdio.h>
#include <stdarg.h>

static TimestampedList<ARecord> list;

#include <time.h>

unsigned long long time_ns(void)
{
	struct timespec tv;

	clock_gettime(CLOCK_MONOTONIC, &tv);
	return tv.tv_sec * 1000000000ULL + tv.tv_nsec;
}


void insert_test(void)
{
	unsigned long long ts;
	unsigned long long t;
	TimestampedList<ARecord>::Iterator it = list.it();

	printf("Insertion test...");
	t = time_ns();
	for (ts = 0; ts < 1000000; ts++) {
		ARecord *r = new ASingleRecord(0, 0);
		it = it.insertAfter(ts, r);
	}
	t = time_ns() - t;
	printf("\t%fms\n", t / 1000000.0);
}

void walk_test(void)
{
	unsigned long long ts;
	unsigned long long t;
	TimestampedList<ARecord>::Iterator it = list.it();

	printf("Walk test...");
	t = time_ns();
	++it;
	ts = 0;
	while (it.valid()) {
		if (it.data()->TimeStamp != ts)
			printf("BUG ts=%Ld readed=%Ld\n", ts,
					it.data()->TimeStamp);
		ts++;
		++it;
	}
	t = time_ns() - t;
	printf("\t%fms\n", t / 1000000.0);
}

void walk_reverse(void)
{
	unsigned long long ts;
	unsigned long long t;
	TimestampedList<ARecord>::Iterator it = list.it();

	printf("Walk test...");
	t = time_ns();
	--it;
	ts = 999999;
	while (it.valid()) {
		if (it.data()->TimeStamp != ts)
			printf("BUG ts=%Ld readed=%Ld\n", ts,
					it.data()->TimeStamp);
		ts--;
		--it;
	}
	t = time_ns() - t;
	printf("\t%fms\n", t / 1000000.0);
}

void after_test(unsigned long long ts)
{
	unsigned long long t;
	unsigned int i;
	TimestampedList<ARecord>::Iterator it = list.it();

	printf("After %Ld test...", ts);
	t = time_ns();
	for (i = 0; i < 10; i++) {
		it = list.it();
		it = it.scanAfter(ts);
		if (!it.valid()) {
			printf("BUG\n");
			abort();
		}
		if (it.data()->TimeStamp != ts)
			printf("BUG ts=%Ld readed=%Ld\n", ts,
					it.data()->TimeStamp);
	}
	t = time_ns() - t;
	printf("\t%fms\n", t / 1000000.0);
}

void clear_test(void)
{
	unsigned long long t;

	printf("Clear test...");
	t = time_ns();
	list.clear();
	t = time_ns() - t;
	printf("\t%fms\n", t / 1000000.0);
}

int main(int argc, char **args)
{
	insert_test();
	walk_test();
	walk_reverse();
	after_test(50);
	after_test(100000);
	after_test(500000);
	after_test(900000);
	clear_test();
	return 0;
}
