//
// C++ Implementation: timescale
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005, 2006  Krzysztof Mazur <krzysiek@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qpainter.h>
#include <time.h>

#include "ar3cmonitor.h"
#include "timescale.h"
#include "commobj.h"
#include "control.h"

TimeScale::TimeScale(QWidget *parent, const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f), font(CO->tsFont())
{
	int h = QFontMetrics(font).height();
	setMaximumHeight(h);
	setMinimumHeight(h);

#if QT_VERSION >= 0x40000
#elif QT_VERSION >= 0x30000
	setBackgroundMode(Qt::NoBackground);
#else
	setBackgroundMode(QWidget::NoBackground);
#endif
}

void TimeScale::paint(QPainter *p)
{
	int w = width();
	int h = QFontMetrics(font).height();

	QColor vgridColor(0, 0, 150, QColor::Hsv);
	
	if (!CO->haveData() || !CO->vgridStep())
		return;
	
	unsigned long long T1 = CO->cursorStamp();
	unsigned long long T0 = T1 - CO->timeRange();
	
	if (!T1) return;

	p->setFont(font);

	QPen pen = p->pen();
	pen.setStyle(Qt::DotLine);
	pen.setColor(vgridColor);
	p->setPen(pen);
	
	unsigned long long step = CO->vgridStep();
	for (unsigned long long T = T0 + step - T0%step; T <= T1; T += step) {
		int x = calculateX(T, T0, w, CO->timeRange());
		time_t tim = T/1000000000ULL;
		struct tm tm;
	
		if (!localtime_r(&tim, &tm))
			return;
		char t[128];
		if (CO->vgridStep() < 60000000000ULL)
			strftime(t, sizeof(t), "%H:%M:%S", &tm);
		else
			strftime(t, sizeof(t), "%H:%M", &tm);
		p->drawText(x-100, 0, 200, h, Qt::AlignHCenter
				| Qt::AlignBottom, QString(t));
	}
}
