/*
 * ar3cctrl - token.c 
 * Copyright (C) 2008  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <token.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <ar3cm_proto.h>

static int genrandom_bytes(void *_data, size_t len)
{
	unsigned char *data = _data;
	size_t i;

	/* FIXME: use better RNG */
	for (i = 0; i < len; i++)
		data[i] = rand() & 0xff;
	return 0;
}

static int token_sha1_init(struct token *t)
{
	t->type = AR3CM_TOKEN_SHA1;
	t->size = 0;
	t->data = NULL;
	return 0;
}

int token_init(struct token *t, unsigned int types, unsigned long long ttl)
{
	t->ttl = ttl;

	if (types & AR3CM_TOKEN_SHA1)
		return token_sha1_init(t);
	return -EINVAL;
}

int token_data_alloc(struct token *t, size_t size)
{
	t->size = size;
	t->data = malloc(t->size);
	if (t->data == NULL)
		return -ENOMEM;
	return 0;
}

int token_generate(struct token *t)
{
	int ret = -EINVAL;

	if (t->type == AR3CM_TOKEN_SHA1) {
		ret = token_data_alloc(t, SHA1_TOKEN_SIZE);
		if (ret)
			goto out;
		genrandom_bytes(t->data, t->size);
		ret = 0;
	}
out:
	return ret;
}

int token_load(struct token *t, void *buf, size_t len)
{
	int ret = -EINVAL;

	if (t->type == AR3CM_TOKEN_SHA1) {
		ret = token_data_alloc(t, len);
		if (ret)
			goto out;
		memcpy(t->data, buf, len);
		ret = 0;
	}
out:
	return ret;
}

struct token *token_alloc(unsigned int types, unsigned long long ttl)
{
	struct token *t;
	int ret;

	t = malloc(sizeof(*t));
	if (t == NULL)
		goto err;
	ret = token_init(t, types, ttl);
	if (ret)
		goto err_free;
	return t;

err_free:
	free(t);
err:
	return NULL;
}

void token_destroy(struct token *t)
{
	if (t->data)
		free(t->data);
}

void token_free(struct token *t)
{
	token_destroy(t);
	free(t);
}

